//Monica Chang
//mjchang@andrew.cmu.edu
//Section D
//Final Project: Capture Red

//WEATHER ELEMENTS
//creating initial empty array for rain
var rain = [];
var raining = true;

//initial background color
var bgColor;

//creating an initial empty array for clouds 
var clouds = []; //highlight clouds
var amountClouds = 30;
var shadowClouds = []; //grey clouds to give clouds depth
var cloudSpeed = 1;

//creating an initial empty array for birds
var birds = [];

//creating array for bird color options

//initial field audio
var rainSound;
var birdSounds;
var captureSound;

//sun elements
var sunX = 0; // sun X position
var sunY = 0; // sun Y position

var init = 0; //initial score/number of birds captured

function preload() {
    //camera lens image
    var lensURL = "https://i.imgur.com/I5OrJoz.png";
    lensPic = loadImage(lensURL);

    //rain sound
    rainSound = loadSound("rainSound.wav");
    rainSound.setVolume(0.5);

    //bird noises
    birdSounds = loadSound("birdSounds.wav");
    birdSounds.setVolume(0.7);

    // camera shutter sound
    captureSound = loadSound("captureSound.wav");
    captureSound.setVolume(1);

}

function setup() {
    createCanvas(640, 440);

    //rain sounds should play first
    rainSound.play();
    rainSound.loop();

    //initial background color, gloomy sky
    bgColor = color(100, 100, 100);

    //initial display of clouds
    for (var i = 0; i < amountClouds; i++) {
        var cloudX = random(width);
        clouds[i] = makeClouds(cloudX);
        shadowClouds[i] = makeShadowClouds(cloudX);
    }

    //initial display of birds
    for (var i = 0; i < 1; i++) {
        var birdsX = random(width);
        birds[i] = makeBirds(birdsX);
    }

    //initial display of the rain
    for (var i = 0; i < 100; i++) {
        rain[i] = new makeRain(random(0, width), random (0, -3000));
    }
}

function draw() {
    background(bgColor);

    //drawing the clouds  
    updateClouds();
    removeClouds();
    addRandomClouds();

    //drawing the birds
    updateBirds();
    removeBirds();
    addRandomBirds();

    //check if raining
    checkRain();

    //drawing the capture screen
    captureScreen();

    //drawing the camera
    drawLens();

    //drawing the score
    displayScore();
    updateScore();

    //end of game display if score reaches 100
    if (init >= 100) {
        displayConclusion();
        //stop sounds when game is over
        if (birdSounds.isPlaying() || rainSound.isPlaying() || captureSound.isPlaying()) {
            rainSound.stop();
            birdSounds.stop();
            captureSound.stop();
        } 
    }
}

function drawLens() {
    image(lensPic, 0, 0);
    tint(115, 255, 150); // displaying the image with a tint

    //initial text instructions for player
    fill("purple");
    stroke("purple");
    strokeWeight(0.5);
    text("Press SPACEBAR to capture the birds and to play sounds.", width / 2 - 60, height - 60);
    text("Press MOUSE to change to a blue sky.", width / 2 - 60, height - 45);    
    noStroke();
    text("Your assignment is to capture 100 photos of these birds.", width / 2 - 60, height - 30);
    text("Good luck, stranger! ", width / 2 - 60, height - 15);
}

//////////////////// BELOW IS THE CODE FOR THE ANIMATION OF THE RAIN \\\\\\\\\\\\\\\\\\\\\\\\\\\\

function checkRain() {
    //meant to check if it is raining or if it is sunny
    if (raining == true) {
        for (i = 0; i < rain.length; i++) {
            rain[i].dropRain();
        }
    }else{
        drawJigglingSun();
    }


}

function mousePressed() {
    raining = false;
    bgColor = color(173, 235, 255);

    //only blue skies let birds sing
    rainSound.stop();
    birdSounds.play();
    birdSounds.loop();
}


function makeRain(x, y) {
    this.x = x;
    this.y = y;
    this.length = 15;

    this.dropRain = function() {
        noStroke();
        fill(255);
        ellipse(this.x, this.y, 3, this.length);
        this.y = this.y + 6;
        if (this.y > 540) { // restart set of rain continuously
            this.y = 0;
        }
        if (this.length < 0) {
            this.length = 0;
        }
    }
}


function drawJigglingSun() {

    fill(245, 187, 87);
    stroke(245, 187, 87);
    push();
    translate(width / 2, height / 3);
    rotate(radians(frameCount / 2));
    ellipse(sunX, sunY, 60, 60);

    //jiggling sun randomly
    sunX = sunX + random(-0.3, 0.3);
    sunY = sunY + random(-0.3, 0.3);

    pop();
}

////////////////////////////// BELOW IS THE CODE FOR THE ANIMATION OF CLOUDS \\\\\\\\\\\\\\\\\\\\\\\\

function updateClouds() {
  //constant update of all cloud positions
    for (var i = 0; i < clouds.length; i++) {
        clouds[i].move();
        clouds[i].display();
    }

    for (var i = 0; i < shadowClouds.length; i++) {
        shadowClouds[i].move();
        shadowClouds[i].display();
    }
}

function addRandomClouds() {
    var newCloud = 0.005;
    if (random(0, 1) < newCloud) {
        clouds.push(makeClouds(width));
        shadowClouds.push(makeShadowClouds(width));
    }
}

function removeClouds() {
    var cloudsToKeep = []; //highlight clouds
    var shadowToKeep = []; //shadow clouds
    for (var i = 0; i < clouds.length; i++) {
        if (clouds[i].x0 + clouds[i].breadth > 0) {
            cloudsToKeep.push(clouds[i]);
        }
    }
    clouds = cloudsToKeep;

    for (var i = 0; i < shadowClouds.length; i++) {
        if (shadowClouds[i].x0 + shadowClouds[i].breadth > 0) {
            shadowToKeep.push(shadowClouds[i]);
        }
    }
    shadowClouds = shadowToKeep;
}


//clouds positions
function cloudsMove() {
    this.x0 += this.speed;
}

function displayClouds() {
    //drawing the moving clouds
    noStroke(); //no outline
    fill(255); //white cloud
    push();
    translate(this.x0, this.y0); //locate cloud body at x0, y0
    ellipse(this.x0, this.y0, this.d * 1.4, this.d / 1.5); //cloud body
    ellipse(this.x0 - 30, this.y0 - 5, this.d * 1.4, this.d / 1.5); //cloud body
    ellipse(this.x0 + 10, this.y0 - 10, this.d * 1.3, this.d);
    ellipse(this.x0 + 30, this.y0 - 10, this.d * 1.3, this.d / 1.5);   
    pop();
}
function displayShadowClouds() {
    //drawing the moving clouds
    noStroke(); //no outline
    fill(255, 255 , 255, 200); //gray cloud
    push();
    translate(this.x0, this.y0 + 20); //locate shadow cloud at x0, y0
    ellipse(this.x0 + 5, this.y0 + 5, this.d * 1.4, this.d / 1.2); //cloud "body"
    ellipse(this.x0 + 5, this.y0 + 10, this.d * 1.3, this.d / 1.2);

    pop();
}

function makeClouds(birthLocationX) {
    var clouds = {x0: birthLocationX,
                y0: random(0, height / 2),
                d: random(20, 80),
                breadth: 100,
                speed: -0.4,
                move: cloudsMove,
                display: displayClouds}
    return clouds;
}
function makeShadowClouds(birthLocationX) {
    var shadowClouds = {x0: birthLocationX,
                        y0: random((height / 2) - 100, height / 2),
                        d: random(40, 100),
                        breadth: 100,
                        speed: -0.4,
                        move: cloudsMove,
                        display: displayShadowClouds}
    return shadowClouds;
}

//////////////////////////// BELOW IS THE CODE FOR THE BIRD ANIMATIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

function updateBirds() {
    //constant update of bird positions
    for (var i = 0; i < birds.length; i++) {
        birds[i].move();
        birds[i].display();
    }
}

function addRandomBirds() {
    var newBirdProbability = 0.003;
    if (random(0,1) < newBirdProbability) {
        birds.push(makeBirds(width));
    }
}

function removeBirds() {

    var birdsToKeep = []; 
    for (var i = 0; i < birds.length; i++) {
        if (birds[i].x + birds[i].breadth > 0) {
            birdsToKeep.push(birds[i]);
        }
    }
    birds = birdsToKeep;

}

function birdsMove() {
    this.x += this.speed;
}

//making the birds
function displayBirds() { 

    var birdHeight = 250;
    noStroke();
    fill(this.c);
    push();
    //the head
    ellipse(this.x, height - birdHeight, this.breadth, 20);
    //the eyes
    fill(0);
    ellipse(this.x, height - birdHeight, 5, 5);

    fill(this.c);
    arc(this.x - 5, -10, 10, 10, 0, PI);

    pop();
}

function makeBirds(birthLocationX) {

    var birds = {x: birthLocationX,
                c: color(random(188, 255), 0, random(0, 255)),
                breadth: 30,
                speed: random(-15.0, -5.0),
                move: birdsMove,
                display: displayBirds}
    return birds;
}

////////////////// BELOW IS THE CODE FOR THE SCORE UPDATES AND DISPLAY \\\\\\\\\\\\\\\\\\\\\\

function displayScore() {
    //documenting the number of birds captured
    stroke("purple");
    strokeWeight(0.5);
    fill("purple");
    textFont("Baskerville");
    text("CAPTURED:" + " " + init , width - 100, height - 70);

}

function captureScreen() {
    //pressing SPACEBAR to indicate the player capturing a bird
    if (keyIsDown(32)) {
        captureSound.play();

        fill(255, 200);
        noStroke();    
        rect(0, 0, width, height); 

    } 

}

function updateScore() {

// points are added when player "captures" bird(s)
    for(var i = 0; i < birds.length; i++) {
        if (keyIsDown(32) && birds[i].x <= width && birds[i].x >= 0) {
            init += 1;
        }
    }

}

function displayConclusion() {

    //when game is over!
    fill(255);
    noStroke();
    rect(0, 0, width, height);

    stroke(2);
    fill(0);
    textAlign(CENTER);
    text("GOOD JOB ON COMPLETING THE ASSIGNMENT, STRANGER!", width / 2, height / 2);
    text("Refresh page to restart.", width / 2, height / 2 + 15);

}
